#import "MYMasterController.h"

@protocol MYMessageServerProtocol

- (void)addMessageClient:(id)aClient;
- (BOOL)removeMessageClient:(id)aClient;
- (void)broadcastMessageString:(NSString *)aString;

@end


@implementation MYMasterController

/*" Actions "*/
- (IBAction)sendMessage:(id)sender
{
	// Braodcast the message and clear compose view
	[server broadcastMessageString:[composeView string]];
	[composeView setString:@""];
}


/*" Append Messages "*/
- (void)appendMessageString:(NSString *)aString
{
	NSRange    appendRange = NSMakeRange([[messageView string] length], 0);
	
	// Append text and scroll if neccessary
	[messageView replaceCharactersInRange:appendRange withString:aString];
	[messageView scrollRangeToVisible:appendRange];
}


- (void)connectionDidDie:(NSNotification *)aNotification
{
	NSLog(@"Error: Connection to server is broken");
}


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	
	server = [[NSConnection rootProxyForConnectionWithRegisteredName:@"server" host:nil] retain];
	
	if(nil == server)
	{
		NSLog(@"Error: Failed to connect to server.");
	}
	else
	{
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(connectionDidDie:)
													 name:NSConnectionDidDieNotification
												   object:nil];
		[server setProtocolForProxy:@protocol(MYMessageServerProtocol)];
		[server addMessageClient:self];
		[server broadcastMessageString:[NSString stringWithFormat:@"Connected: %@ %d\n",
			[[NSProcessInfo processInfo] processName],
			[[NSProcessInfo processInfo] processIdentifier]]];
	}
}


- (void)applicationWillTerminate:(NSNotification *)aNotification
{
	if(NO == [server removeMessageClient:self])
	{
		NSLog(@"Error: Failed to remove client.");
	}
	[server release];
	server = nil;
}


@end
